/***************************************************************************//**
 *   @file   AD5421.c
 *   @brief  Implementation of AD5421 Driver.
 *   @author Mihai Bancisor
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 594
*******************************************************************************/

/******************************************************************************/
/* Include Files                                                              */
/******************************************************************************/
#include "AD5421.h"		// AD5421 definitions.

/***************************************************************************//**
 * @brief Initializes the SPI communication peripheral and resets the part.
 *
 * @return status.
*******************************************************************************/
unsigned char AD5421_Init(void)
{
	unsigned char status = 0x1;
    
	SPI_Init(0, 1000000, 1, 1);
	// setup 
	AD5421_SetRegisterValue(AD5421_REG_CTRL,
							AD5421_CTRL_WATCHDOG_DISABLE |
							AD5421_CTRL_ADC_SOURCE_TEMP |
							AD5421_CTRL_ADC_ENABLE);	
	if(AD5421_GetRegisterValue(AD5421_REG_CTRL) != 0x1180)
    {
		status = 0x0;
	}
	 
	return (status);
}

/***************************************************************************//**
 * @brief Writes the value to the register
 *
 * @param -  regAddress - The address of the register to write to.
 * @param -  regValue - The value to write to the register.
 *
 * @return  None.    
*******************************************************************************/
void AD5421_SetRegisterValue(unsigned char regAddress, unsigned short regValue)
{
	unsigned char data[5] = {0x03, 0x00, 0x00, 0x00, 0x00};	
	
	data[1] = regAddress;
	data[2] = (unsigned char)((regValue & 0xFF00) >> 8);
	data[3] = (unsigned char)((regValue & 0x00FF) >> 0);
	ADI_CS_LOW;	    
	SPI_Write(data,3);
	ADI_CS_HIGH;
}

/***************************************************************************//**
 * @brief Reads the value of the selected register
 *
 * @param regAddress - The address of the register to read.
 *
 * @return data - The value of the selected register register.
*******************************************************************************/
unsigned long AD5421_GetRegisterValue(unsigned char regAddress)
{
	unsigned char data[5] = {0x03, 0x00, 0x00, 0x00, 0x00};
	unsigned long receivedData = 0x00;	
	
	data[1] = regAddress | AD5421_READ;
	ADI_CS_LOW;  
	SPI_Write(data,3);
	ADI_CS_HIGH;
	ADI_CS_LOW;
	SPI_Read(data,3);
	ADI_CS_HIGH;
	receivedData += (data[1] << 8);
	receivedData += (data[2] << 0);

    return receivedData;
}

/***************************************************************************//**
 * @brief Writes to the DAC register.
 *
 * @param value - The value to be vritten to the DAC register.
 *
 * @return None.
*******************************************************************************/
void AD5421_WriteDac(unsigned short value)
{
	AD5421_SetRegisterValue(AD5421_REG_DAC_DATA, value);
}

/***************************************************************************//**
 * @brief Read the temperature of the die.
 *
 * @return temp - die temp in degrees C.
*******************************************************************************/
unsigned char AD5421_GetTemperature(void)
{
	unsigned char temp = 0;
	temp = (unsigned char)AD5421_GetRegisterValue(AD5421_REG_FAULT) & 0xFF;
	temp = temp - 128;
	temp = temp * 1.771;
	temp = 125 - temp;
	
	return temp;
	
}